@@ -12,6 +12,7 @@ import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.extensions.toPercent
 import com.unciv.ui.screens.victoryscreen.RankingType
+import kotlin.math.ceil
 import kotlin.math.min
 import kotlin.math.pow
 import kotlin.math.sqrt
@@ -61,6 +62,12 @@ class TradeEvaluation {
     }
 
     fun getTradeAcceptability(trade: Trade, evaluator: Civilization, tradePartner: Civilization): Int {
+        val citiesAskedToSurrender = trade.ourOffers.filter { it.type == TradeType.City }.count()
+        val maxCitiesToSurrender = ceil(evaluator.cities.size.toFloat() / 5).toInt()
+        if (citiesAskedToSurrender > maxCitiesToSurrender) {
+            return Int.MIN_VALUE
+        }
+
         val sumOfTheirOffers = trade.theirOffers.asSequence()
                 .filter { it.type != TradeType.Treaty } // since treaties should only be evaluated once for 2 sides
                 .map { evaluateBuyCost(it, evaluator, tradePartner) }.sum()
@@ -70,7 +77,7 @@ class TradeEvaluation {
         // If we're making a peace treaty, don't try to up the bargain for people you don't like.
         // Leads to spartan behaviour where you demand more, the more you hate the enemy...unhelpful
         if (trade.ourOffers.none { it.name == Constants.peaceTreaty || it.name == Constants.researchAgreement }) {
-            val relationshipLevel = evaluator.getDiplomacyManager(tradePartner).relationshipLevel()
+            val relationshipLevel = evaluator.getDiplomacyManager(tradePartner).relationshipIgnoreAfraid()
             if (relationshipLevel == RelationshipLevel.Enemy) sumOfOurOffers = (sumOfOurOffers * 1.5).toInt()
             else if (relationshipLevel == RelationshipLevel.Unforgivable) sumOfOurOffers *= 2
         }
@@ -105,15 +112,14 @@ class TradeEvaluation {
             }
 
             TradeType.Strategic_Resource -> {
-                val resources = civInfo.getCivResourcesByName()
-                val amountWillingToBuy = 2 - resources[offer.name]!!
+                val amountWillingToBuy = 2 - civInfo.getResourceAmount(offer.name)
                 if (amountWillingToBuy <= 0) return 0 // we already have enough.
                 val amountToBuyInOffer = min(amountWillingToBuy, offer.amount)
 
                 val canUseForBuildings = civInfo.cities
-                        .any { city -> city.cityConstructions.getBuildableBuildings().any { it.getResourceRequirements().containsKey(offer.name) } }
+                        .any { city -> city.cityConstructions.getBuildableBuildings().any { it.getResourceRequirementsPerTurn().containsKey(offer.name) } }
                 val canUseForUnits = civInfo.cities
-                        .any { city -> city.cityConstructions.getConstructableUnits().any { it.getResourceRequirements().containsKey(offer.name) } }
+                        .any { city -> city.cityConstructions.getConstructableUnits().any { it.getResourceRequirementsPerTurn().containsKey(offer.name) } }
                 if (!canUseForBuildings && !canUseForUnits) return 0
 
                 return 50 * amountToBuyInOffer
@@ -193,7 +199,7 @@ class TradeEvaluation {
             }
             TradeType.Luxury_Resource -> {
                 return when {
-                    civInfo.getCivResourcesByName()[offer.name]!! > 1 -> 250 // fair price
+                    civInfo.getResourceAmount(offer.name) > 1 -> 250 // fair price
                     civInfo.hasUnique(UniqueType.RetainHappinessFromLuxury) -> // If we retain 50% happiness, value at 375
                         750 - (civInfo.getMatchingUniques(UniqueType.RetainHappinessFromLuxury)
                             .first().params[0].toPercent() * 250).toInt()
@@ -210,12 +216,11 @@ class TradeEvaluation {
                 if (!civInfo.isAtWar()) return 50 * offer.amount
 
                 val canUseForUnits = civInfo.gameInfo.ruleset.units.values
-                    .any { it.getResourceRequirements().containsKey(offer.name)
+                    .any { it.getResourceRequirementsPerTurn().containsKey(offer.name)
                             && it.isBuildable(civInfo) }
                 if (!canUseForUnits) return 50 * offer.amount
 
-                val amountLeft = civInfo.getCivResourcesByName()[offer.name]
-                    ?: throw Exception("Got a strategic resource offer but the the resource doesn't exist in this ruleset!")
+                val amountLeft = civInfo.getResourceAmount(offer.name)
 
                 // Each strategic resource starts costing 100 more when we ass the 5 resources baseline
                 // That is to say, if I have 4 and you take one away, that's 200
@@ -250,16 +255,15 @@ class TradeEvaluation {
                 val city = civInfo.cities.firstOrNull { it.id == offer.name }
                     ?: throw Exception("Got an offer to sell city id " + offer.name + " which does't seem to exist for this civ!")
 
-                val capitalcity = civInfo.getCapital()!!
-                val distanceCost = distanceCityTradeModifier(civInfo, capitalcity, city)
+                val distanceBonus = distanceCityTradeModifier(civInfo, city)
                 val stats = city.cityStats.currentCityStats
                 val sumOfStats =
-                    stats.culture + stats.gold + stats.science + stats.production + stats.happiness + stats.food - distanceCost
-                return sumOfStats.toInt() * 100
+                    stats.culture + stats.gold + stats.science + stats.production + stats.happiness + stats.food + distanceBonus
+                return min(sumOfStats.toInt() * 100, 1000)
             }
             TradeType.Agreement -> {
                 if (offer.name == Constants.openBorders) {
-                    return when (civInfo.getDiplomacyManager(tradePartner).relationshipLevel()) {
+                    return when (civInfo.getDiplomacyManager(tradePartner).relationshipIgnoreAfraid()) {
                         RelationshipLevel.Unforgivable -> 10000
                         RelationshipLevel.Enemy -> 2000
                         RelationshipLevel.Competitor -> 500
@@ -272,11 +276,15 @@ class TradeEvaluation {
         }
     }
 
-    private fun distanceCityTradeModifier(civInfo: Civilization, capitalcity: City, city: City): Int{
-        val distanceBetweenCities = capitalcity.getCenterTile().aerialDistanceTo(city.getCenterTile())
+    /** This code returns a positive value if the city is significantly far away from the capital
+     * and given how this method is used this ends up making such cities more expensive. That's how
+     * I found it. I'm not sure it makes sense. One might also find arguments why cities closer to
+     * the capital are more expensive. */
+    private fun distanceCityTradeModifier(civInfo: Civilization, city: City): Int{
+        val distanceToCapital = civInfo.getCapital()!!.getCenterTile().aerialDistanceTo(city.getCenterTile())
 
-        if (distanceBetweenCities < 500) return 0
-        return min(50,  (500 - distanceBetweenCities) * civInfo.getEraNumber())
+        if (distanceToCapital < 500) return 0
+        return (distanceToCapital - 500) * civInfo.getEraNumber()
     }
 
     fun evaluatePeaceCostForThem(ourCivilization: Civilization, otherCivilization: Civilization): Int {
@@ -287,11 +295,34 @@ class TradeEvaluation {
         if (ourCombatStrength > theirCombatStrength) {
             val absoluteAdvantage = ourCombatStrength - theirCombatStrength
             val percentageAdvantage = absoluteAdvantage / theirCombatStrength.toFloat()
+            // We don't add the same constraint here. We should not make peace easily if we're
+            // heavily advantaged.
             return (absoluteAdvantage * percentageAdvantage).toInt() * 10
         } else {
+            // This results in huge values for large power imbalances. However, we should not give
+            // up everything just because there is a big power imbalance. There's a better chance to
+            // recover if you don't give away all your cities for example.
+            //
+            // Example A (this would probably give us away everything):
+            // absoluteAdvantage = 10000 - 100 = 9500
+            // percentageAdvantage = 9500 / 100 = 95
+            // return -(9500 * 95) * 10 = -9025000
+            //
+            // Example B (this is borderline)
+            // absoluteAdvantage = 10000 - 2500 = 7500
+            // percentageAdvantage = 7500 / 2500 = 3
+            // return -(7500 * 3) * 10 = -225000
+            //
+            // Example C (this is fine):
+            // absoluteAdvantage = 10000 - 5000 = 5000
+            // percentageAdvantage = 5000 / 5000 = 1
+            // return -(5000 * 1) * 10 = -50000
+            //
+            // Hence we cap the max cost at 100k which equals about 2 or 3 cities in the mid game
+            // (stats ~30 each)
             val absoluteAdvantage = theirCombatStrength - ourCombatStrength
             val percentageAdvantage = absoluteAdvantage / ourCombatStrength.toFloat()
-            return -(absoluteAdvantage * percentageAdvantage).toInt() * 10
+            return -min((absoluteAdvantage * percentageAdvantage).toInt() * 10, 100000)
         }
     }
 
