@@ -27,7 +27,7 @@ class TileImprovement : RulesetStatsObject() {
     override fun getUniqueTarget() = UniqueTarget.Improvement
     val shortcutKey: Char? = null
     // This is the base cost. A cost of 0 means created instead of buildable.
-    val turnsToBuild: Int = 0
+    val turnsToBuild: Int = -1
 
 
     fun getTurnsToBuild(civInfo: Civilization, unit: MapUnit): Int {
@@ -75,8 +75,10 @@ class TileImprovement : RulesetStatsObject() {
 
     fun handleImprovementCompletion(builder: MapUnit) {
         val tile = builder.getTile()
+
         if (hasUnique(UniqueType.TakesOverAdjacentTiles))
             UnitActions.takeOverTilesAround(builder)
+
         if (tile.resource != null) {
             val city = builder.getTile().getCity()
             if (city != null) {
@@ -85,6 +87,7 @@ class TileImprovement : RulesetStatsObject() {
                 city.civ.cache.updateCivResources()
             }
         }
+
         if (hasUnique(UniqueType.RemovesFeaturesIfBuilt)) {
             // Remove terrainFeatures that a Worker can remove
             // and that aren't explicitly allowed under the improvement
@@ -100,6 +103,8 @@ class TileImprovement : RulesetStatsObject() {
 
             tile.setTerrainFeatures(tile.terrainFeatures.filterNot { it in removableTerrainFeatures })
         }
+
+        tile.owningCity?.reassignPopulationDeferred()
     }
 
     /**
@@ -121,7 +126,7 @@ class TileImprovement : RulesetStatsObject() {
             "All" -> true
             "All Road" -> isRoad()
             "Great Improvement", "Great" -> isGreatImprovement()
-            in uniques -> true
+            in uniqueMap -> true
             else -> false
         }
     }
@@ -264,7 +269,7 @@ class TileImprovement : RulesetStatsObject() {
 
         return ruleset.units.values.asSequence()
             .filter { unit ->
-                turnsToBuild != 0
+                turnsToBuild != -1
                     && unit.getMatchingUniques(UniqueType.BuildImprovements, StateForConditionals.IgnoreConditionals)
                         .any { matchesBuildImprovementsFilter(it.params[0]) }
                 || unit.hasUnique(UniqueType.CreateWaterImprovements)
@@ -275,8 +280,7 @@ class TileImprovement : RulesetStatsObject() {
     private fun getCreatingUnits(ruleset: Ruleset): List<BaseUnit> {
         return ruleset.units.values.asSequence()
             .filter { unit ->
-                (unit.getMatchingUniques(UniqueType.ConstructImprovementConsumingUnit, StateForConditionals.IgnoreConditionals)
-                        + unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly))
+                unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly, StateForConditionals.IgnoreConditionals)
                     .any { it.params[0] == name }
             }.toList()
     }
