@@ -2,9 +2,17 @@ package com.unciv.ui.screens.mapeditorscreen
 
 import com.badlogic.gdx.Application
 import com.badlogic.gdx.Gdx
+import com.badlogic.gdx.files.FileHandle
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.graphics.Texture
+import com.badlogic.gdx.graphics.g2d.TextureRegion
+import com.badlogic.gdx.scenes.scene2d.Group
+import com.badlogic.gdx.scenes.scene2d.Touchable
+import com.badlogic.gdx.scenes.scene2d.ui.Image
+import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable
 import com.unciv.UncivGame
 import com.unciv.logic.map.MapParameters
+import com.unciv.logic.map.MapShape
 import com.unciv.logic.map.MapSize
 import com.unciv.logic.map.MapSizeNew
 import com.unciv.logic.map.TileMap
@@ -18,12 +26,16 @@ import com.unciv.ui.screens.mapeditorscreen.tabs.MapEditorOptionsTab
 import com.unciv.ui.popups.ConfirmPopup
 import com.unciv.ui.components.tilegroups.TileGroup
 import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.components.KeyCharAndCode
-import com.unciv.ui.components.KeyboardPanningListener
+import com.unciv.ui.components.input.KeyCharAndCode
+import com.unciv.ui.components.input.KeyShortcutDispatcherVeto
+import com.unciv.ui.components.input.KeyboardPanningListener
+import com.unciv.ui.images.ImageWithCustomSize
+import com.unciv.ui.popups.ToastPopup
 import com.unciv.ui.screens.basescreen.RecreateOnResize
 import com.unciv.ui.screens.worldscreen.ZoomButtonPair
 import com.unciv.utils.Concurrency
 import com.unciv.utils.Dispatcher
+import com.unciv.utils.Log
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.Job
 
@@ -125,6 +137,34 @@ class MapEditorScreen(map: TileMap? = null): BaseScreen(), RecreateOnResize {
 
     fun getToolsWidth() = stage.width * 0.4f
 
+    fun setWorldWrap(newValue: Boolean) {
+        if (newValue == tileMap.mapParameters.worldWrap) return
+        setWorldWrapFixOddWidth(newValue)
+        if (newValue && overlayFile != null) {
+            overlayFile = null
+            ToastPopup("An overlay image is incompatible with world wrap and was deactivated.", stage, 4000)
+            tabs.options.update()
+        }
+        recreateMapHolder()
+    }
+
+    private fun setWorldWrapFixOddWidth(newValue: Boolean) = tileMap.mapParameters.run {
+        // Turning *off* WW and finding an odd width means it must have been rounded
+        // down by the TileMap constructor - fix so we can turn it back on later
+        if (worldWrap && mapSize.width % 2 != 0 && shape == MapShape.rectangular)
+            mapSize.width--
+        worldWrap = newValue
+    }
+
+    private fun recreateMapHolder(actionWhileRemoved: ()->Unit = {}) {
+        val savedScale = mapHolder.scaleX
+        clearOverlayImages()
+        mapHolder.remove()
+        actionWhileRemoved()
+        mapHolder = newMapHolder()
+        mapHolder.zoom(savedScale)
+    }
+
     private fun newMapHolder(): EditorMapHolder {
         ImageGetter.setNewRuleset(ruleset)
         // setNewRuleset is missing some graphics - those "EmojiIcons"&co already rendered as font characters
@@ -138,18 +178,20 @@ class MapEditorScreen(map: TileMap? = null): BaseScreen(), RecreateOnResize {
         tileMap.setStartingLocationsTransients()
         UncivGame.Current.translations.translationActiveMods = ruleset.mods
 
-        val result = EditorMapHolder(this, tileMap) {
+        val newHolder = EditorMapHolder(this, tileMap) {
             tileClickHandler?.invoke(it)
         }
         for (oldPanningListener in stage.root.listeners.filterIsInstance<KeyboardPanningListener>())
             stage.removeListener(oldPanningListener)  // otherwise they accumulate
-        result.mapPanningSpeed = UncivGame.Current.settings.mapPanningSpeed
-        stage.addListener(KeyboardPanningListener(result, allowWASD = false))
+        newHolder.mapPanningSpeed = UncivGame.Current.settings.mapPanningSpeed
+        stage.addListener(KeyboardPanningListener(newHolder, allowWASD = false))
         if (Gdx.app.type == Application.ApplicationType.Desktop)
-            result.isAutoScrollEnabled = UncivGame.Current.settings.mapAutoScroll
+            newHolder.isAutoScrollEnabled = UncivGame.Current.settings.mapAutoScroll
 
-        stage.root.addActorAt(0, result)
-        stage.scrollFocus = result
+        addOverlayToMapHolder(newHolder.actor as Group)  // That's the initially empty Group ZoomableScrollPane allocated
+
+        stage.root.addActorAt(0, newHolder)
+        stage.scrollFocus = newHolder
 
         isDirty = true
         modsTabNeedsRefresh = true
@@ -157,15 +199,19 @@ class MapEditorScreen(map: TileMap? = null): BaseScreen(), RecreateOnResize {
         naturalWondersNeedRefresh = true
 
         if (UncivGame.Current.settings.showZoomButtons) {
-            zoomController = ZoomButtonPair(result)
+            zoomController = ZoomButtonPair(newHolder)
             zoomController!!.setPosition(10f, 10f)
             stage.addActor(zoomController)
         }
 
-        return result
+        return newHolder
     }
 
+    // We contain a map...
+    override fun getShortcutDispatcherVetoer() = KeyShortcutDispatcherVeto.createTileGroupMapDispatcherVetoer()
+
     fun loadMap(map: TileMap, newRuleset: Ruleset? = null, selectPage: Int = 0) {
+        clearOverlayImages()
         mapHolder.remove()
         tileMap = map
         ruleset = newRuleset ?: RulesetCache.getComplexRuleset(map.mapParameters)
@@ -181,11 +227,12 @@ class MapEditorScreen(map: TileMap? = null): BaseScreen(), RecreateOnResize {
         }
 
     fun applyRuleset(newRuleset: Ruleset, newBaseRuleset: String, mods: LinkedHashSet<String>) {
-        mapHolder.remove()
-        tileMap.mapParameters.baseRuleset = newBaseRuleset
-        tileMap.mapParameters.mods = mods
-        tileMap.ruleset = newRuleset
-        ruleset = newRuleset
+        recreateMapHolder {
+            tileMap.mapParameters.baseRuleset = newBaseRuleset
+            tileMap.mapParameters.mods = mods
+            tileMap.ruleset = newRuleset
+            ruleset = newRuleset
+        }
         mapHolder = newMapHolder()
         modsTabNeedsRefresh = false
     }
@@ -251,4 +298,74 @@ class MapEditorScreen(map: TileMap? = null): BaseScreen(), RecreateOnResize {
             job.cancel()
         jobs.clear()
     }
+
+    //region Overlay Image
+
+    // To support world wrap with an overlay, one could maybe do up to tree versions of the same
+    // Image tiled side by side (therefore "clearOverlayImages"), they _could_ use the same Texture
+    // instance - that part works. But how to position and clip them properly escapes me - better
+    // coders are welcome to try. To work around, we simply turn world wrap off when an overlay is
+    // loaded, and allow to freely turn WW on and off. After all, the distinction becomes relevant
+    // *only* when a game is started, units move, and tile neighbors get a meaning.
+
+    private var imageOverlay: Image? = null
+
+    internal var overlayFile: FileHandle? = null
+        set(value) {
+            field = value
+            overlayFileChanged(value)
+        }
+
+    internal var overlayAlpha = 0.33f
+        set(value) {
+            field = value
+            overlayAlphaChanged(value)
+        }
+
+    private fun clearOverlayImages() {
+        val oldImage = imageOverlay ?: return
+        imageOverlay = null
+        oldImage.remove()
+        (oldImage.drawable as? TextureRegionDrawable)?.region?.texture?.dispose()
+    }
+
+    private fun overlayFileChanged(value: FileHandle?) {
+        clearOverlayImages()
+        if (value == null) return
+        if (tileMap.mapParameters.worldWrap) {
+            setWorldWrapFixOddWidth(false)
+            ToastPopup("World wrap is incompatible with an overlay and was deactivated.", stage, 4000)
+            tabs.options.update()
+        }
+        recreateMapHolder()
+    }
+
+    private fun overlayAlphaChanged(value: Float) {
+        imageOverlay?.color?.a = value
+    }
+
+    private fun addOverlayToMapHolder(newHolderContent: Group) {
+        clearOverlayImages()
+        if (overlayFile == null) return
+
+        try {
+            val texture = Texture(overlayFile)
+            texture.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear)
+            imageOverlay = ImageWithCustomSize(TextureRegion(texture))
+        } catch (ex: Throwable) {
+            Log.error("Invalid overlay image", ex)
+            overlayFile = null
+            ToastPopup("Invalid overlay image", stage, 3000)
+            tabs.options.update()
+            return
+        }
+
+        imageOverlay?.apply {
+            touchable = Touchable.disabled
+            setFillParent(true)
+            color.a = overlayAlpha
+            newHolderContent.addActor(this)
+        }
+    }
+    //endregion
 }
